@@ -297,9 +297,12 @@ brrweights<-function(strata,psu, match=NULL, small=c("fail","split","merge"),
 ## Designs with replication weights rather than survey structure.
 ##
 
-as.svrepdesign<- function(design,type=c("auto","JK1","JKn","BRR","bootstrap","subbootstrap","mrbbootstrap","Fay"),
+as.svrepdesign<- function(design,...) UseMethod("as.svrepdesign")
+as.svrepdesign.default<-function(design,type=c("auto","JK1","JKn","BRR","bootstrap","subbootstrap","mrbbootstrap","Fay"),
                           fay.rho=0, fpc=NULL, fpctype=NULL,...,compress=TRUE, mse=getOption("survey.replicates.mse")){
-
+    if(!is.null(design$postStrata)){
+        stop("postStratify, rake, or calibrate the design *after* adding replicate weights")
+        }
   type<-match.arg(type)
 
   if (type=="auto"){
@@ -432,7 +435,8 @@ as.svrepdesign<- function(design,type=c("auto","JK1","JKn","BRR","bootstrap","su
 svrepdesign<-function(variables, repweights, weights,data=NULL,...) UseMethod("svrepdesign",data)
 
 svrepdesign.default<-function(variables=NULL,repweights=NULL, weights=NULL,
-                              data=NULL,type=c("BRR","Fay","JK1", "JKn","bootstrap","other"),
+                              data=NULL,type=c("BRR","Fay","JK1", "JKn","bootstrap",
+                                               "ACS","successive-difference","JK2","other"),
                               combined.weights=TRUE, rho=NULL, bootstrap.average=NULL,
                               scale=NULL,rscales=NULL,fpc=NULL, fpctype=c("fraction","correction"),
                               mse=getOption("survey.replicates.mse"),...)
@@ -443,7 +447,7 @@ svrepdesign.default<-function(variables=NULL,repweights=NULL, weights=NULL,
   if(type=="Fay" && is.null(rho))
     stop("With type='Fay' you must supply the correct rho")
   
-  if (type %in% c("JK1","JKn")  && !is.null(rho))
+  if (type %in% c("JK1","JKn","ACS","successive-difference","JK2")  && !is.null(rho))
     warning("rho not relevant to JK1 design: ignored.")
   
   if (type %in% c("other")  && !is.null(rho))
@@ -503,10 +507,16 @@ svrepdesign.default<-function(variables=NULL,repweights=NULL, weights=NULL,
   if (!is.null(rscales) && !(length(rscales) %in% c(1, ncol(repweights)))){
     stop(paste("rscales has length ",length(rscales),", should be ncol(repweights)",sep=""))
   }
+
+    if (type %in% c("ACS", "successive-difference")){
+        if(!is.null(scale) | !is.null(rscales))
+            warning(paste("with type",type,"scale= and rscales= are not needed and will be ignored"))
+    }
   
     if (type == "BRR"){
         ## the default, so check it hasn't been accidentally defaulted to
         if (!is.null(scale)) warning("type='BRR' does not use 'scale=' argument")
+        if (!is.null(rho))  warning("type='BRR' does not use 'rho=' argument, you may want type='Fay'")
       scale<-1/ncol(repweights)
       }
   if (type=="Fay")
@@ -538,6 +548,18 @@ svrepdesign.default<-function(variables=NULL,repweights=NULL, weights=NULL,
       rscales<-1/apply(repweights,2,max)
     } else stop("Must provide rscales for combined JKn weights")
 
+    if (type %in% c("ACS","successive-difference")){
+        rscales<-rep(1, ncol(repweights))
+        scale<-4/ncol(repweights)
+    }
+
+    if (type =="JK2"){
+        warning(paste("with type",type,"scale= and rscales= are not needed and will be ignored"))
+        rscales<-rep(1, ncol(repweights))
+        scale<-1
+    }
+    
+
   if (type=="other" && (is.null(rscales) || is.null(scale))){
     if (is.null(rscales)) rscales<-rep(1,NCOL(repweights))
     if (is.null(scale)) scale<-1
@@ -548,14 +570,15 @@ svrepdesign.default<-function(variables=NULL,repweights=NULL, weights=NULL,
   if (!is.null(fpc)){
       if (missing(fpctype)) stop("Must specify fpctype")
       fpctype<-match.arg(fpctype)
-      if (type %in% c("BRR","Fay")) stop("fpc not available for this type")
+      if (type %in% c("BRR","Fay","JK2","ACS","successive-difference")) stop("fpc not available for this type")
       if (type %in% "bootstrap") stop("Separate fpc not needed for bootstrap")
       if (length(fpc)!=length(rscales)) stop("fpc is wrong length")
       if (any(fpc>1) || any(fpc<0)) stop("Illegal fpc value")
       fpc<-switch(fpctype,correction=fpc,fraction=1-fpc)
       rscales<-rscales*fpc
   }
-  
+
+    if (is.null(scale)) scale<-1
   
   rval<-list(type=type, scale=scale, rscales=rscales,  rho=rho,call=sys.call(),
              combined.weights=combined.weights)
@@ -678,194 +701,6 @@ weights.survey.design<-function(object,...){
 }
 
 
-svyquantile.svyrep.design<-svrepquantile<-function(x,design,quantiles,method="linear",
-                                                   interval.type=c("probability","quantile"),f=1,
-                                                   return.replicates=FALSE,
-                                                   ties=c("discrete","rounded"),na.rm=FALSE,...){
-
-  if (!exists(".Generic",inherits=FALSE))
-    .Deprecated("svyquantile")
-
-  ties<-match.arg(ties)
-  interval<-match.arg(interval.type)
-  if (design$type %in% c("JK1","JKn") && interval=="quantile")
-    warning("Jackknife replicate weights may not give valid standard errors for quantiles")
-  if (design$type %in% "other" && interval=="quantile")
-    warning("Not all replicate weight designs give valid standard errors for quantiles.")
-  if (inherits(x,"formula"))
-		x<-model.frame(x,design$variables,na.action=if(na.rm) na.pass else na.fail)
-    else if(typeof(x) %in% c("expression","symbol"))
-        x<-eval(x, design$variables)
-
-
-   if (na.rm){
-         nas<-rowSums(is.na(x))
-       design<-design[nas==0,]
-        if (length(nas)>length(design$prob))
-          x<-x[nas==0,,drop=FALSE]
-        else
-          x[nas>0,]<-0
-      }
-    
-  if (NROW(x)<=1){
-      rval<-matrix(rep(as.matrix(x),length(quantiles)),ncol=NCOL(x),nrow=length(quantiles),byrow=TRUE)
-      dimnames(rval)<-list(paste("q",round(quantiles,2),sep=""), names(x))
-      if (getOption("survey.drop.replicates") && !is.null(design$selfrep) && all(design$selfrep))
-          vv<-matrix(0,ncol=NCOL(x),nrow=length(quantiles))
-      else
-          vv<-matrix(NA,ncol=NCOL(x),nrow=length(quantiles))
-      dimnames(vv)<-list(paste("q",round(quantiles,2),sep=""), names(x))
-      attr(rval,"var")<-vv
-      attr(rval,"statistic")<-quantiles
-      if (return.replicates)
-          rval<-list(mean=rval,replicates=NULL)
-      class(rval)<-"svrepstat"
-      return(rval)
-  }
-
-  
-    w<-weights(design,"analysis")
-
-    if (interval=="quantile"){
-      ## interval on quantile scale
-      if (ties=="discrete")
-        computeQuantiles<-function(xx){
-          oo<-order(xx)
-          
-          ws<-weights(design,"sampling")
-          cum.ws<-cumsum(ws[oo])/sum(ws)
-          rval<-approx(cum.ws,xx[oo],method=method,f=f,
-                       yleft=min(xx),yright=max(xx),
-                       xout=quantiles,ties=min)$y
-          
-          cum.w<-apply(w,2,function(wi) cumsum(wi[oo])/sum(wi))
-          
-          qq<-apply(cum.w, 2,function(cum.wi) approx(cum.wi,xx[oo],method=method,f=f,
-                                                     yleft=min(xx),yright=max(xx),
-                                                     xout=quantiles,ties=min)$y)
-          if (length(quantiles)>1)
-            qq<-t(qq)
-          else
-            qq<-as.matrix(qq)
-          ##rval<-colMeans(qq)
-          
-          rval<-list(quantiles=rval,
-                     variances=diag(as.matrix(svrVar(qq,design$scale,design$rscales,mse=design$mse,coef=rval))))
-          if (return.replicates)
-            rval<-c(rval, list(replicates=qq))
-          rval
-        } else { ##ties="rounded"
-          computeQuantiles<-function(xx){
-            ws<-weights(design,"sampling")
-
-            wws<-rowsum(ws,xx,reorder=TRUE)
-            uxx<-sort(unique(xx))
-            
-            cum.wws<-cumsum(wws)/sum(wws)
-            rval<-approx(cum.wws,uxx,method=method,f=f,
-                     yleft=min(xx),yright=max(xx),
-                         xout=quantiles,ties=min)$y
-            
-            cum.w<-apply(rowsum(w,xx,reorder=TRUE),2,function(wi) cumsum(wi)/sum(wi))
-            
-            qq<-apply(cum.w, 2,function(cum.wi) approx(cum.wi,uxx,method=method,f=f,
-                                                       yleft=min(xx),yright=max(xx),
-                                                       xout=quantiles,ties=min)$y)
-            if (length(quantiles)>1)
-              qq<-t(qq)
-            else
-              qq<-as.matrix(qq)
-            ##rval<-colMeans(qq)
-            
-            rval<-list(quantiles=rval,
-                       variances=diag(as.matrix(svrVar(qq,design$scale,design$rscales,mse=design$mse,coef=rval))))
-            if (return.replicates)
-              rval<-c(rval, list(replicates=qq))
-            rval
-          }
-        }
-    } else {
-      ## interval on probability scale, backtransformed.
-      if (ties=="discrete"){
-        computeQuantiles<-function(xx){
-          oo<-order(xx)
-          w<-weights(design,"sampling")
-          cum.w<- cumsum(w[oo])/sum(w)
-          Qf<-approxfun(cum.w,xx[oo],method=method,f=f,
-                        yleft=min(xx),yright=max(xx),
-                        ties=min)
-          
-          point.estimates<-Qf(quantiles)
-          if(length(quantiles)==1)
-            estfun<-as.numeric(xx<point.estimates)
-          else
-            estfun<-0+outer(xx,point.estimates,"<")
-          est<-svymean(estfun,design, return.replicates=return.replicates)
-          if (return.replicates)
-            q.estimates<-matrix(Qf(est$replicates),nrow=NROW(est$replicates))
-          ci<-matrix(Qf(c(coef(est)+2*SE(est), coef(est)-2*SE(est))),ncol=2)
-          variances<-((ci[,1]-ci[,2])/4)^2
-          rval<-list(quantiles=point.estimates,
-                     variances=variances)
-          if (return.replicates)
-            rval<-c(rval, list(replicates=q.estimates))
-          rval
-        }
-      } else {
-        ## ties=rounded
-        computeQuantiles<-function(xx){
-          w<-weights(design,"sampling")
-          ww<-rowsum(w,xx,reorder=TRUE)
-          uxx<-sort(unique(xx))
-          cum.w<- cumsum(ww)/sum(ww)
-          Qf<-approxfun(cum.w,uxx,method=method,f=f,
-                        yleft=min(xx),yright=max(xx),
-                        ties=min)
-          
-          point.estimates<-Qf(quantiles)
-          if(length(quantiles)==1)
-            estfun<-as.numeric(xx<point.estimates)
-          else
-                estfun<-0+outer(xx,point.estimates,"<")
-          est<-svymean(estfun, design, return.replicates=return.replicates)
-          if (return.replicates)
-            q.estimates<-matrix(Qf(est$replicates),nrow=NROW(est$replicates))
-          ci<-matrix(Qf(c(coef(est)+2*SE(est), coef(est)-2*SE(est))),ncol=2)
-          variances<-((ci[,1]-ci[,2])/4)^2
-          rval<-list(quantiles=point.estimates,
-                     variances=variances)
-          if (return.replicates)
-            rval<-c(rval, list(replicates=q.estimates))
-          rval
-        }
-        
-      }
-    }
-
-  if (!is.null(dim(x)))
-    results<-apply(x,2,computeQuantiles)
-  else
-    results<-computeQuantiles(x)
-  
-  rval<-matrix(sapply(results,"[[","quantiles"),ncol=NCOL(x),nrow=length(quantiles),
-               dimnames=list(paste("q",round(quantiles,2),sep=""), names(x)))
-  vv<-matrix(sapply(results,"[[","variances"),ncol=NCOL(x),nrow=length(quantiles),
-             dimnames=list(paste("q",round(quantiles,2),sep=""), names(x)))
-  attr(rval,"var")<-vv
-  attr(rval, "statistic")<-"quantiles"
-  if (return.replicates) {
-    reps<-do.call(cbind,lapply(results,"[[","replicates"))
-    attr(reps,"scale")<-design$scale
-    attr(reps,"rscales")<-design$rscales
-    attr(reps,"mse")<-design$mse
-    rval<-list(mean=rval, replicates=reps)
-  }
-  class(rval)<-"svrepstat"
-  rval
-  
-}
-
-
 svrVar<-function(thetas, scale, rscales,na.action=getOption("na.action"),mse=getOption("survey.replicates.mse"),coef){
   thetas<-get(na.action)(thetas)
   naa<-attr(thetas,"na.action")
@@ -1235,7 +1070,9 @@ svycoxph.svyrep.design<-function(formula, design, subset=NULL,rescale=NULL,...,r
     stop("all variables must be in design= argument")
   .survey.prob.weights<-pwts
   full<-with(data,eval(g))
-  
+  if (inherits(full, "coxph.penal"))
+      warning("svycoxph does not support penalised terms")
+ 
   nas<-attr(full$model, "na.action")
   
   betas<-matrix(ncol=length(coef(full)),nrow=ncol(design$repweights))
@@ -1322,7 +1159,10 @@ svrepglm<-svyglm.svyrep.design<-function(formula, design, subset=NULL,family=sta
     .Deprecated("svyglm")
   
   subset<-substitute(subset)
-  subset<-eval(subset, design$variables, parent.frame())
+    subset<-eval(subset, design$variables, parent.frame())
+    if (any(is.na(subset)))
+        stop("subset must not contain NA values")
+
   if (!is.null(subset))
     design<-design[subset,]
 
@@ -1432,6 +1272,7 @@ svrepglm<-svyglm.svyrep.design<-function(formula, design, subset=NULL,family=sta
     full$replicates<-betas
   }  
   class(full)<-c("svrepglm", "svyglm", class(full))
+    
   full$call<-sys.call(-1)
   if(!("formula" %in% names(full$call))) {
     if (is.null(names(full$call)))
@@ -1471,7 +1312,8 @@ print.summary.svyglm<-function (x, digits = max(3, getOption("digits") - 3),
         }
         printCoefmat(coefs, digits = digits, signif.stars = signif.stars, 
             na.print = "NA", ...)
-    
+    if (x$df.resid<=0)
+        cat("\nZero or negative residual df; p-values not defined\n")
     cat("\n(Dispersion parameter for ", x$family$family, " family taken to be ", 
         format(x$dispersion), ")\n\n",  "Number of Fisher Scoring iterations: ", 
         x$iter, "\n", sep = "")
@@ -1597,7 +1439,7 @@ svyratio.svyrep.design<-svrepratio<-function(numerator=formula,denominator, desi
         vars[i,j]<-svrVar(allstats$replicates[,i]/allstats$replicates[,nn+j],
                           design$scale, design$rscales,mse=design$mse,coef=rval$ratio[i,j])
         if (deff=="replace" || deff)
-          deffs[i,j]<-deff(svytotal(numerator[,i]-rval[i,j]*denominator[,j],design,deff=deff))
+          deffs[i,j]<-deff(svytotal(numerator[,i]-rval$ratio[i,j]*denominator[,j],design,deff=deff))
       }
     }
   }
@@ -1657,6 +1499,16 @@ logLik.svrepglm<-function(object,...){
    stop("svrepglm not fitted by maximum likelihood.")
 }
 
+vcov.svyrep.design<-function (object, replicates, centre, ...) 
+{
+    if (object$mse) 
+        svrVar(replicates, scale = object$scale, rscales = object$rscales, 
+            mse = object$mse, coef=centre)
+    else svrVar(replicates, scale = object$scale, rscales = object$rscales, 
+        mse = object$mse)
+}
+
+
 
 withReplicates<-function(design, theta,  ..., return.replicates=FALSE){
   UseMethod("withReplicates",design)
@@ -2042,7 +1894,7 @@ postStratify.svyrep.design<-function(design, strata, population,
                            match(designlabel, both$label),
                            reorder=TRUE)
     repreweight<-  both$Pop.Freq/replicateFreq
-    design$repweights <- as.matrix(design$repweights)*repreweight[index]
+    design$repweights <- as.matrix(design$repweights)*repreweight[index,]
   } else { 
     replicateFreq<- rowsum(as.matrix(design$repweights)*oldpw,
                            match(designlabel, both$label),
